// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO168/16aio168_Linux_2.x.x.x_DN/driver/irq.c $
// $Rev: 53667 $
// $Date: 2023-08-31 16:09:35 -0500 (Thu, 31 Aug 2023) $

// 16AIO168: Device Driver: source file

#include "main.h"



//*****************************************************************************
void irq_icr_mod(dev_data_t* dev, u32 value, u32 mask)
{
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_icr_32, value, mask);
}



//*****************************************************************************
void dev_irq_isr_local_handler(dev_data_t* dev)
{
	#define	ICR_IRQ0_REQUEST	0x008
	#define	ICR_IRQ1_REQUEST	0x080
	#define	ICR_IRQ2_REQUEST	0x800

	int	handled	= 0;
	u32	icr;
	u32	irq;

	icr	= os_reg_mem_rx_u32(NULL, dev->vaddr.gsc_icr_32);

	if (icr & ICR_IRQ0_REQUEST)
	{
		handled	= 1;
		icr		&= ~ (u32) ICR_IRQ0_REQUEST;

		// Resume any blocked threads.
		irq	= GSC_FIELD_DECODE(icr, 2, 0);

		switch (irq)
		{
			default:

				gsc_wait_resume_irq_main(dev, GSC_WAIT_MAIN_SPURIOUS);
				break;

			case AIO168_IRQ0_IDLE_INIT:

				gsc_wait_resume_irq_gsc(dev, AIO168_WAIT_GSC_IDLE_INIT);
				break;

			case AIO168_IRQ0_AUTOCAL_DONE:

				gsc_wait_resume_irq_gsc(dev, AIO168_WAIT_GSC_AUTOCAL_DONE);
				break;
		}
	}
	else if (dev->cache.fw_rev > 0x903)
	{
		icr	|= ICR_IRQ0_REQUEST;
	}

	if (icr & ICR_IRQ1_REQUEST)
	{
		// Clear the interrupt.
		handled	= 1;
		icr		&= ~ (u32) ICR_IRQ1_REQUEST;

		// Resume any blocked threads.
		irq	= GSC_FIELD_DECODE(icr, 6, 4);

		switch (irq)
		{
			default:

				gsc_wait_resume_irq_main(dev, GSC_WAIT_MAIN_SPURIOUS);
				break;

			case AIO168_IRQ1_AI_BUF_THR_L2H:

				gsc_wait_resume_irq_gsc(dev, AIO168_WAIT_GSC_AI_BUF_THR_L2H);
				break;

			case AIO168_IRQ1_AI_BUF_THR_H2L:

				gsc_wait_resume_irq_gsc(dev, AIO168_WAIT_GSC_AI_BUF_THR_H2L);
				break;
		}

	}
	else if (dev->cache.fw_rev > 0x903)
	{
		icr	|= ICR_IRQ1_REQUEST;
	}

	if (icr & ICR_IRQ2_REQUEST)
	{
		// Clear the interrupt.
		handled	= 1;
		icr		&= ~ (u32) ICR_IRQ2_REQUEST;

		// Resume any blocked threads.
		irq	= GSC_FIELD_DECODE(icr, 10, 8);

		switch (irq)
		{
			default:

				gsc_wait_resume_irq_main(dev, GSC_WAIT_MAIN_SPURIOUS);
				break;

			case AIO168_IRQ2_AO_BUF_THR_L2H:

				gsc_wait_resume_irq_gsc(dev, AIO168_WAIT_GSC_AO_BUF_THR_L2H);
				break;

			case AIO168_IRQ2_AO_BUF_THR_H2L:

				gsc_wait_resume_irq_gsc(dev, AIO168_WAIT_GSC_AO_BUF_THR_H2L);
				break;

			case AIO168_IRQ2_AO_BURST_DONE:

				gsc_wait_resume_irq_gsc(dev, AIO168_WAIT_GSC_AO_BURST_DONE);
				break;
		}
	}
	else if (dev->cache.fw_rev > 0x903)
	{
		icr	|= ICR_IRQ2_REQUEST;
	}

	if (handled)
	{
		os_reg_mem_tx_u32(NULL, dev->vaddr.gsc_icr_32, icr);
	}
	else
	{
		// We don't know the source of the interrupt.
		gsc_wait_resume_irq_main(dev, GSC_WAIT_MAIN_SPURIOUS);
	}
}



//*****************************************************************************
int dev_irq_create(dev_data_t* dev)
{
	int	ret;

	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_icr_32, 0);
	ret	= gsc_irq_create(dev);
	return(ret);
}



//*****************************************************************************
void dev_irq_destroy(dev_data_t* dev)
{
	if (dev->vaddr.gsc_icr_32)
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_icr_32, 0);

	gsc_irq_destroy(dev);
}


