// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO168/16aio168_Linux_2.x.x.x_DN/samples/aout/main.c $
// $Rev: 53652 $
// $Date: 2023-08-31 15:52:24 -0500 (Thu, 31 Aug 2023) $

// 16AIO168: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _parse_args(int argc, char** argv, args_t* args)
{
	const gsc_arg_item_t	list[]	=
	{
		//	type				var						arg			values					desc
		{	GSC_ARG_S32_FLAG,	&args->io_mode,			"-bmdma",	{ GSC_IO_MODE_BMDMA },	"Transfer data using Block Mode DMA."					},
		{	GSC_ARG_S32_FLAG,	&args->continuous,		"-c",		{ -1 },					"Continue testing until an error occurs."				},
		{	GSC_ARG_S32_RANGE,	&args->chan_qty,		"-c",		{ 1, 8 },				"Generate output on # channels (default: 4)."			},
		{	GSC_ARG_S32_FLAG,	&args->continuous,		"-C",		{ +1 },					"Continue testing even if errors occur."				},
		{	GSC_ARG_S32_MIN,	&args->empty_qty,		"-e",		{ 1 },					"Stop if the buffer runs empty # times (required -ml)."	},
		{	GSC_ARG_S32_FLAG,	&args->indefinate,		"-i",		{ 1 },					"Transmit data indefinately, til there's an issue."		},
		{	GSC_ARG_S32_MIN,	&args->log_qty,			"-l",		{ 1 },					"Stop after logging # entries (requires -ml)."			},
		{	GSC_ARG_S32_MIN,	&args->minute_limit,	"-m",		{ 1 },					"Continue testing for up to # minutes."					},
		{	GSC_ARG_S32_FLAG,	&args->monitor,			"-ml",		{ 1 },					"Monitor and log performance data (" LOG_FILE ")."		},
		{	GSC_ARG_S32_RANGE,	&args->minutes,			"-M",		{ 1, 10080 },			"Output data for # minutes (max: 7 days)."				},
		{	GSC_ARG_S32_MIN,	&args->test_limit,		"-n",		{ 1 },					"Continue testing for up to # iterations."				},
		{	GSC_ARG_S32_FLAG,	&args->io_mode,			"-pio",		{ GSC_IO_MODE_PIO },	"Transfer data using PIO (default)."					},
		{	GSC_ARG_S32_RANGE,	&args->fsamp,			"-r",		{ 1, 300000 },			"Output data at # S/S (default: 50,000)."				},
		{	GSC_ARG_S32_FLAG,	&args->reg_dump,		"-rd",		{ 1 },					"When -e, -l# or -us# kick in, print a register dump."	},
		{	GSC_ARG_S32_RANGE,	&args->seconds,			"-s",		{ 1, 3600 },			"Output data for # seconds (default: 15)."				},
		{	GSC_ARG_S32_MIN,	&args->scan_limit,		"-sl",		{ 1 },					"Limit output to # scans per write call."				},
		{	GSC_ARG_S32_MIN,	&args->us_stop,			"-us",		{ 1 },					"Stop if write takes # us or less (requires -ml)."		},
		{	GSC_ARG_DEV_INDEX,	&args->index,			"index",	{ 1 },					"The zero based index of the device to access."			}
	};

	const gsc_arg_set_t	set	=
	{
		/* name	*/	"aout",
		/* desc	*/	"Analog Output Sample Application",
		/* qty	*/	SIZEOF_ARRAY(list),
		/* list	*/	list
	};

	int	errs;

	memset(args, 0, sizeof(args_t));
	args->chan_qty	= 4;
	args->fsamp		= 50000;
	args->io_mode	= GSC_IO_MODE_PIO;
	args->seconds	= 15;

	errs	= gsc_args_parse(argc, argv, &set);

	return(errs);
}



//*****************************************************************************
static void _show_access_index(const args_t* args)
{
	gsc_label("Accessing Device");
	printf("%d\n", args->index);
}



//*****************************************************************************
static void _show_time_stamp(void)
{
	const char*	psz;
	struct tm*	stm;
	time_t		tt;

	time(&tt);
	stm	= localtime(&tt);
	psz	= (char*) asctime(stm);
	gsc_label("Time Stamp");
	printf("%s", psz);
}



/******************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Control the overall flow of the application.
*
*	Arguments:
*
*		argc			The number of command line arguments.
*
*		argv			The list of command line arguments.
*
*	Returned:
*
*		EXIT_SUCCESS	No errors were encounterred.
*		EXIT_FAILURE	One or more problems were encounterred.
*
******************************************************************************/

int main(int argc, char** argv)
{
	args_t	args;
	int		errs;
	time_t	exec		= time(NULL);
	long	failures	= 0;
	long	hours;
	long	mins;
	time_t	now;
	long	passes		= 0;
	int		ret;
	long	secs;
	time_t	t_limit;
	time_t	test;
	long	tests		= 0;

	for (;;)	// A convenience loop.
	{
		gsc_label_init(33);
		test	= time(NULL);
		errs	= _parse_args(argc, argv, &args);

		// Introduction

		t_limit	= exec + (args.minute_limit * 60);
		os_id_host();
		_show_time_stamp();
		_show_access_index(&args);
		errs	+= aio168_init_util(1);
		errs	+= os_id_driver(aio168_open, aio168_read, aio168_close);
		errs	+= aio168_count_boards(1, &args.qty);

		if (args.qty <= 0)
			errs++;

		// Open access to device

		if (errs == 0)
			errs	= aio168_open_util(args.index, 0, -1, 1, &args.fd);

		if (errs)
			args.continuous	= 0;

		if (errs == 0)
		{
			// Device identification

			errs	= aio168_id_device(args.fd, -1, 1);

			// Perform testing

			errs	+= perform_tests(&args);

			// Close device access

			errs	+= aio168_close_util(args.fd, -1, 1);
		}

		// End of processing.

		now	= time(NULL);
		tests++;

		if (errs)
		{
			failures++;
			printf(	"\nRESULTS: FAIL <---  (%d error%s)",
					errs,
					(errs == 1) ? "" : "s");
		}
		else
		{
			passes++;
			printf("\nRESULTS: PASS");
		}

		secs	= (long) (now - test);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(" (duration %ld:%ld:%02ld)\n", hours, mins, secs);

		secs	= (long) (now - exec);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(	"SUMMARY: tests %ld, pass %ld, fail %ld"
				" (duration %ld:%ld:%02ld)\n\n",
				tests,
				passes,
				failures,
				hours,
				mins,
				secs);

		if (args.continuous == 0)
			break;

		if ((args.continuous < 0) && (errs))
			break;

		if ((args.minute_limit > 0) && (now >= t_limit))
			break;

		if ((args.test_limit > 0) && (tests >= args.test_limit))
			break;
	}

	ret	= ((args.qty <= 0) || (errs) || (failures)) ? EXIT_FAILURE : EXIT_SUCCESS;
	return(ret);
}


