// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO168/16aio168_Linux_2.x.x.x_DN/samples/burstout/perform.c $
// $Rev: 53653 $
// $Date: 2023-08-31 15:52:41 -0500 (Thu, 31 Aug 2023) $

// 16AIO168: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static	u32		_data[8 * 1024];
static	long	_qty;	// Number of samples to send per block.



//*****************************************************************************
static int _output_buffer_status(const args_t* args, int* empty)
{
	int	errs	= 0;
	s32	lvl;
	int	ret;
	s32	sts;

	lvl			= 0;
	ret			= aio168_ioctl(args->fd, AIO168_IOCTL_AO_BUF_THR_LVL, &lvl);
	errs		+= ret ? 1 : 0;

	ret			= aio168_ioctl(args->fd, AIO168_IOCTL_AO_BUF_THR_STS, &sts);
	errs		+= ret ? 1 : 0;
	empty[0]	= (sts == AIO168_AO_BUF_THR_STS_CLEAR) ? 1 : 0;
	return(errs);
}



//*****************************************************************************
static int _io_write(const args_t* args, int errs)
{
	int				empty;
	os_time_ns_t	end;
	os_time_ns_t	now;
	char*			ptr;
	int				send;
	int				sent;

	gsc_label("Outputting Data");

	for (;;)	// A convenience loop.
	{
		if (errs)
		{
			errs	= 0;
			printf("SKIPPED  (due to errors)\n");
			break;
		}

		errs	= 0;

		// Write the data to the device.
		os_time_get_ns(&end);
		end.tv_sec	+= args->seconds;

		for (;;)
		{
			send	= _qty * 4;
			ptr		= (char*) _data;

			for (;;)
			{
				sent	= aio168_write(args->fd, ptr, send);

				if ((sent < 0) || (sent == send))
					break;

				ptr		+= sent;
				send	-= sent;

				os_time_get_ns(&now);

				if (((now.tv_sec > end.tv_sec) ||
					((now.tv_sec == end.tv_sec) && (now.tv_nsec >= end.tv_nsec))))
				{
					break;
				}
			}

			if (sent < 0)
				break;

			os_time_get_ns(&now);

			if (((now.tv_sec > end.tv_sec) ||
				((now.tv_sec == end.tv_sec) && (now.tv_nsec >= end.tv_nsec))))
			{
				break;
			}
		}

		// Wait for the output buffer to empty.

		for (;;)
		{
			errs	= _output_buffer_status(args, &empty);

			if ((errs) || (empty))
				break;
		}

		// Compute the results.

		if (sent < 0)
		{
			errs	= 1;
			printf("FAIL <---  (write error)\n");
		}
		else
		{
			printf("PASS  (%d seconds)\n", args->seconds);
		}

		break;
	}

	return(errs);
}



//*****************************************************************************
static int _init_data(long rate, int chan_qty)
{
	#define	PIE		(3.1415962)

	u32*		dest;
	int			i;
	u32			max;
	long		period_len;	// The length of the pattern in samples.
	long		period_tot;	// Pattern samples total for all channels.
	long		period;		// Pattern length in x units.
	int			samp;
	const char*	units;

	gsc_label("Initializing Data");
	max		= (0x1UL << 16) - 1;
	memset(_data, 0, sizeof(_data));

	period_len	= 20;
	period_tot	= period_len * chan_qty;
	period		= (long) (1000000000. / rate * period_len);
	units		= "ns";

	if ((period % 1000) == 0)
	{
		period	/= 1000;
		units	= "us";
	}

	if ((period % 1000) == 0)
	{
		period	/= 1000;
		units	= "ms";
	}

	_qty	= period_tot;

	// Initialize the data buffer.
	// The data is encoded in OFFSET BINARY format.
	dest	= _data;;

	for (i = 0, samp = 0; i < _qty; i += chan_qty, samp++)
	{
		samp	= (samp < period_len) ? samp : 0;

		// 1st channel: sawtooth wave
		*dest++	= ((u16) (max - (max * samp / (period_len - 1))))
				| 0x00000L;	// channel tag

		// 2nd channel: ramp wave
		*dest++	= ((u16) (max * samp / (period_len - 1)))
				| 0x10000L;	// channel tag

		// 3rd channel: square wave
		*dest++	= ((samp < (period_len / 2)) ? max : 0)
				| 0x20000L;	// channel tag

		// 4th channel: sine wave
		*dest++	= (u32) (((long) (max >> 1)) * (1 + sin((2 * PIE / period_len) * samp)))
				| 0x30000L	// channel tag
				| 0x80000;	// group end
	}

	dest[-1]	|= 0x100000;	// burst end

	printf("DONE  (pattern length: %ld samples, %ld %s)\n", period_len, period, units);
	return(0);
}



//*****************************************************************************
static int _save_data(int errs)
{
	FILE*		file;
	int			i;
	long		l;
	const char*	name	= "burst_data.txt";
	long		samples	= _qty;

	gsc_label("Saving");

	for (;;)
	{
		if (errs)
		{
			printf("SKIPPED  (errors)\n");
			errs	= 0;
			break;
		}

		file	= fopen(name, "w+b");

		if (file == NULL)
		{
			printf("FAIL <---  (%d. unable to create %s)\n", __LINE__, name);
			errs	= 1;
			break;
		}

		for (l = 0; l < samples; l++)
		{
			i	= fprintf(file, "  %08lX", (long) _data[l]);

			if (i != 10)
			{
				printf("FAIL <---  (%d. fprintf() failure on %s)\n", __LINE__, name);
				errs	= 1;
				break;
			}

			if ((_data[l] & 0x80000))
			{
				i	= fprintf(file, "  GEF");

				if (i != 5)
				{
					printf("FAIL <---  (%d. fprintf() failure on %s)\n", __LINE__, name);
					errs	= 1;
					break;
				}
			}

			if ((_data[l] & 0x100000))
			{
				i	= fprintf(file, "  BEF");

				if (i != 5)
				{
					printf("FAIL <---  (%d. fprintf() failure on %s)\n", __LINE__, name);;
					errs	= 1;
					break;
				}
			}

			if ((_data[l] & 0x80000))
			{
				i	= fprintf(file, "\r\n");

				if (i != 2)
				{
					printf("FAIL <---  (%d. fprintf() failure on %s)\n", __LINE__, name);
					errs	= 1;
					break;
				}
			}
		}

		fclose(file);

		if (errs == 0)
			printf("PASS  (%s)\n", name);

		break;
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int		chan_qty	= 4;
	int		errs		= 0;
	long	rate		= 300000;

	gsc_label("Analog Output");
	printf("\n");
	gsc_label_level_inc();

	errs	+= _init_data				(rate, chan_qty);
	errs	+= aio168_config_ao			(args->fd, -1, 1, rate);
	errs	+= aio168_tx_io_timeout		(args->fd, -1, 1, 10, NULL);
	errs	+= aio168_rag_nrate			(args->fd, -1, 1, 3000, NULL);
	errs	+= aio168_rag_enable		(args->fd, -1, 1, AIO168_GEN_ENABLE_YES, NULL);
	errs	+= aio168_ao_burst_clk_src	(args->fd, -1, 1, AIO168_AO_BURST_CLK_SRC_RAG, NULL);
	errs	+= aio168_ext_sync_src		(args->fd, -1, 1, AIO168_AI_EXT_SYNC_SRC_AOS, NULL);
	errs	+= aio168_ao_burst_enable	(args->fd, -1, 1, AIO168_AO_BURST_ENABLE_YES, NULL);

	gsc_label("Output Channels");
	printf("%d\n", chan_qty);

	errs	+= _save_data				(errs);
	errs	+= _io_write				(args, errs);

	gsc_label_level_dec();
	return(errs);
}


