// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO168/16aio168_Linux_2.x.x.x_DN/samples/irq/irq1.c $
// $Rev: 50469 $
// $Date: 2022-02-22 11:59:34 -0600 (Tue, 22 Feb 2022) $

// 16AIO168: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _low_2_high_ai_buf_thr_lvl_dec(int fd)
{
	int	errs	= 0;
	int	ret;
	s32	val;

	// With default settings the FIFO will fill in about 1/3 second.
	val		= 0x7FFF;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_AI_BUF_THR_LVL, &val);
	errs	+= ret ? 1 : 0;

	val		= AIO168_GEN_ENABLE_YES;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_RAG_ENABLE, &val);
	errs	+= ret ? 1 : 0;

	os_sleep_ms(200);

	// Select the interrupt then reduce the threshold level.

	val		= AIO168_IRQ1_AI_BUF_THR_L2H;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_IRQ1_SEL, &val);
	errs	+= ret ? 1 : 0;

	val		= 1;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_AI_BUF_THR_LVL, &val);
	errs	+= ret ? 1 : 0;

	return(errs);
}



//*****************************************************************************
static int _low_2_high_add_data(int fd)
{
	int	errs	= 0;
	int	ret;
	s32	val;

	val		= 9830;	// should take about 1/10 of a second
	ret		= aio168_ioctl(fd, AIO168_IOCTL_AI_BUF_THR_LVL, &val);
	errs	+= ret ? 1 : 0;

	val		= AIO168_IRQ1_AI_BUF_THR_L2H;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_IRQ1_SEL, &val);
	errs	+= ret ? 1 : 0;

	val		= AIO168_GEN_ENABLE_YES;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_RAG_ENABLE, &val);
	errs	+= ret ? 1 : 0;

	// Allow plenty of time for the interrupt.
	os_sleep_ms(200);
	return(errs);
}



//*****************************************************************************
static int _irq1_ai_buf_thr_sts_l2h_test(int fd)
{
	int	errs	= 0;
	int	ret;

	gsc_label("AI Buffer Threshold Status Low-to-High");
	printf("\n");
	gsc_label_level_inc();

	gsc_label("AI Buffer Threshold Level Decrease");
	ret		= aio168_ioctl(fd, AIO168_IOCTL_INITIALIZE, NULL);
	errs	+= ret ? 1 : 0;
	errs	+= irq_wait_test(fd, AIO168_WAIT_GSC_AI_BUF_THR_L2H, _low_2_high_ai_buf_thr_lvl_dec);

	gsc_label("AI Add Input Data");
	ret		= aio168_ioctl(fd, AIO168_IOCTL_INITIALIZE, NULL);
	errs	+= ret ? 1 : 0;
	errs	+= irq_wait_test(fd, AIO168_WAIT_GSC_AI_BUF_THR_L2H, _low_2_high_add_data);

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static int _high_2_low_setup(int fd)
{
	int	errs	= 0;
	int	i;
	int	ret;
	s32	val;

	val		= 10;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_AI_BUF_THR_LVL, &val);
	errs	+= ret ? 1 : 0;

	val		= AIO168_AI_SCAN_CLK_SRC_BCR;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_AI_SCAN_CLK_SRC, &val);
	errs	+= ret ? 1 : 0;

	for (i = 0; i < 10; i++)
	{
		ret		= aio168_ioctl(fd, AIO168_IOCTL_AI_SYNC, NULL);
		errs	+= ret ? 1 : 0;
		os_sleep_ms(10);
		ret		= aio168_ioctl(fd, AIO168_IOCTL_AI_BUF_THR_STS, &val);
		errs	+= ret ? 1 : 0;

		if (val == AIO168_AI_BUF_THR_STS_SET)
			break;
	}

	val		= AIO168_GEN_ENABLE_NO;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_RAG_ENABLE, &val);
	errs	+= ret ? 1 : 0;

	val		= AIO168_IRQ1_AI_BUF_THR_H2L;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_IRQ1_SEL, &val);
	errs	+= ret ? 1 : 0;
	return(errs);
}



//*****************************************************************************
static int _high_2_low_ai_buf_thr_lvl_inc(int fd)
{
	int	errs	= 0;
	int	i;
	u32	reg;
	int	ret;
	s32	val;

	errs	+= _high_2_low_setup(fd);

	for (i = 0; (errs == 0) && (i < 100); i++)
		errs	+= aio168_reg_read(fd, -1, 0, AIO168_GSC_IDBR, &reg);

	val		= 0x7FFF;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_AI_BUF_THR_LVL, &val);
	errs	+= ret ? 1 : 0;

	// Allow plenty of time for the threshold level to clear.
	os_sleep_ms(1);
	return(errs);
}



//*****************************************************************************
static int _high_2_low_ai_read_data(int fd)
{
	u32		buf[1024];
	int		errs	= 0;
	int		i;
	int		ret;
	s32		val;

	errs	+= _high_2_low_setup(fd);

	val		= GSC_IO_MODE_PIO;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_RX_IO_MODE, &val);
	errs	+= ret ? 1 : 0;

	val		= 0;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_RX_IO_TIMEOUT, &val);
	errs	+= ret ? 1 : 0;

	for (i = 0; i <= 1000; i++)
		aio168_read(fd, buf, sizeof(buf));

	// Allow plenty of time for the interrupt.
	os_sleep_ms(250);
	return(errs);
}



//*****************************************************************************
static int _high_2_low_ai_buf_clear(int fd)
{
	int	errs	= 0;
	int	ret;

	errs	+= _high_2_low_setup(fd);

	ret		= aio168_ioctl(fd, AIO168_IOCTL_AI_BUF_CLEAR, NULL);
	errs	+= ret ? 1 : 0;

	// Allow plenty of time for the threshold level to clear.
	os_sleep_ms(1);
	return(errs);
}



//*****************************************************************************
static int _irq1_ai_buf_thr_sts_h2l_test(int fd)
{
	int	errs	= 0;
	int	ret;

	gsc_label("AI Buffer Threshold Status High-to-Low");
	printf("\n");
	gsc_label_level_inc();

	gsc_label("AI Buffer Threshold Level Increase");
	ret		= aio168_ioctl(fd, AIO168_IOCTL_INITIALIZE, NULL);
	errs	+= ret ? 1 : 0;
	errs	+= irq_wait_test(fd, AIO168_WAIT_GSC_AI_BUF_THR_H2L, _high_2_low_ai_buf_thr_lvl_inc);

	gsc_label("AI Buffer Read Data");
	ret		= aio168_ioctl(fd, AIO168_IOCTL_INITIALIZE, NULL);
	errs	+= ret ? 1 : 0;
	errs	+= irq_wait_test(fd, AIO168_WAIT_GSC_AI_BUF_THR_H2L, _high_2_low_ai_read_data);

	gsc_label("AI Buffer Clear");
	ret		= aio168_ioctl(fd, AIO168_IOCTL_INITIALIZE, NULL);
	errs	+= ret ? 1 : 0;
	errs	+= irq_wait_test(fd, AIO168_WAIT_GSC_AI_BUF_THR_H2L, _high_2_low_ai_buf_clear);

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
int irq1_test(int fd)
{
	int	errs	= 0;

	gsc_label("IRQ1");
	printf("\n");
	gsc_label_level_inc();

	errs	+= _irq1_ai_buf_thr_sts_l2h_test(fd);
	errs	+= _irq1_ai_buf_thr_sts_h2l_test(fd);

	gsc_label_level_dec();
	return(errs);
}



