// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO168/16aio168_Linux_2.x.x.x_DN/samples/irq/irq2.c $
// $Rev: 50469 $
// $Date: 2022-02-22 11:59:34 -0600 (Tue, 22 Feb 2022) $

// 16AIO168: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _low_2_high_setup(int fd)
{
	int			errs	= 0;
	int			i;
	gsc_reg_t	reg;
	int			ret;
	s32			val;

	val		= 0x1FF;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_AO_BUF_THR_LVL, &val);
	errs	+= ret ? 1 : 0;

	val		= AIO168_GEN_ENABLE_NO;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_RBG_ENABLE, &val);
	errs	+= ret ? 1 : 0;

	val		= AIO168_IRQ2_AO_BUF_THR_L2H;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_IRQ2_SEL, &val);
	errs	+= ret ? 1 : 0;

	for (i = 0; i < 300; i++)
	{
		reg.reg		= AIO168_GSC_ODBR;
		reg.value	= 0x88000;
		ret			= aio168_ioctl(fd, AIO168_IOCTL_REG_WRITE, &reg);
		errs		+= ret ? 1 : 0;
	}

	return(errs);
}



//*****************************************************************************
static int _low_2_high_ao_buf_thr_lvl_dec(int fd)
{
	int	errs	= 0;
	int	ret;
	s32	val;

	errs	+= _low_2_high_setup(fd);

	val		= 1;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_AO_BUF_THR_LVL, &val);
	errs	+= ret ? 1 : 0;

	os_sleep_ms(10);
	return(errs);
}



//*****************************************************************************
static int _low_2_high_ao_add_output_data(int fd)
{
	int			errs	= 0;
	int			i;
	gsc_reg_t	reg;
	int			ret;

	errs	+= _low_2_high_setup(fd);

	for (i = 0; i < 1024; i++)
	{
		reg.reg		= AIO168_GSC_ODBR;
		reg.value	= 0x88000;
		ret			= aio168_ioctl(fd, AIO168_IOCTL_REG_WRITE, &reg);
		errs		+= ret ? 1 : 0;
	}

	os_sleep_ms(10);
	return(errs);
}



//*****************************************************************************
static int _irq2_ao_buf_thr_sts_l2h_test(int fd)
{
	int	errs	= 0;
	int	ret;

	gsc_label("AO Buffer Threshold Status Low-to-High");
	printf("\n");
	gsc_label_level_inc();

	gsc_label("AO Buffer Threshold Level Decrease");
	ret		= aio168_ioctl(fd, AIO168_IOCTL_INITIALIZE, NULL);
	errs	+= ret ? 1 : 0;
	errs	+= irq_wait_test(fd, AIO168_WAIT_GSC_AO_BUF_THR_L2H, _low_2_high_ao_buf_thr_lvl_dec);

	gsc_label("AO Add Output Data");
	ret		= aio168_ioctl(fd, AIO168_IOCTL_INITIALIZE, NULL);
	errs	+= ret ? 1 : 0;
	errs	+= irq_wait_test(fd, AIO168_WAIT_GSC_AO_BUF_THR_L2H, _low_2_high_ao_add_output_data);

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static int _high_to_low_setup(int fd)
{
	int			errs	= 0;
	int			i;
	gsc_reg_t	reg;
	int			ret;
	s32			val;

	val		= 100;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_AO_BUF_THR_LVL, &val);
	errs	+= ret ? 1 : 0;

	val		= AIO168_GEN_ENABLE_NO;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_RBG_ENABLE, &val);
	errs	+= ret ? 1 : 0;

	for (i = 0; i < 1024; i++)
	{
		reg.reg		= AIO168_GSC_ODBR;
		reg.value	= 0x88000;
		ret			= aio168_ioctl(fd, AIO168_IOCTL_REG_WRITE, &reg);
		errs		+= ret ? 1 : 0;
	}

	val		= AIO168_IRQ2_AO_BUF_THR_H2L;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_IRQ2_SEL, &val);
	errs	+= ret ? 1 : 0;
	return(errs);
}



//*****************************************************************************
static int _high_to_low_ao_buf_clear(int fd)
{
	int	errs	= 0;
	int	ret;

	errs	+= _high_to_low_setup(fd);

	ret		= aio168_ioctl(fd, AIO168_IOCTL_AO_BUF_CLEAR, NULL);
	errs	+= ret ? 1 : 0;
	os_sleep_ms(50);
	return(errs);
}



//*****************************************************************************
static int _high_to_low_ao_buf_thr_lvl_inc(int fd)
{
	int	errs	= 0;
	int	ret;
	s32	val;

	errs	+= _high_to_low_setup(fd);

	val		= 0x7FFF;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_AO_BUF_THR_LVL, &val);
	errs	+= ret ? 1 : 0;
	os_sleep_ms(50);
	return(errs);
}



//*****************************************************************************
static int _high_to_low_ao_output(int fd)
{
	int	errs	= 0;
	int	ret;
	s32	val;

	errs	+= _high_to_low_setup(fd);

	val		= AIO168_GEN_ENABLE_YES;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_RBG_ENABLE, &val);
	errs	+= ret ? 1 : 0;
	os_sleep_ms(50);
	return(errs);
}



//*****************************************************************************
static int _irq2_ao_buf_thr_sts_h2l_test(int fd)
{
	int	errs	= 0;
	int	ret;

	gsc_label("AO Buffer Threshold Status High-to-Low");
	printf("\n");
	gsc_label_level_inc();

	gsc_label("AO Buffer Clear");
	ret		= aio168_ioctl(fd, AIO168_IOCTL_INITIALIZE, NULL);
	errs	+= ret ? 1 : 0;
	errs	+= irq_wait_test(fd, AIO168_WAIT_GSC_AO_BUF_THR_H2L, _high_to_low_ao_buf_clear);

	gsc_label("AO Buffer Threshold Level Increase");
	ret		= aio168_ioctl(fd, AIO168_IOCTL_INITIALIZE, NULL);
	errs	+= ret ? 1 : 0;
	errs	+= irq_wait_test(fd, AIO168_WAIT_GSC_AO_BUF_THR_H2L, _high_to_low_ao_buf_thr_lvl_inc);

	gsc_label("AO Output");
	ret		= aio168_ioctl(fd, AIO168_IOCTL_INITIALIZE, NULL);
	errs	+= ret ? 1 : 0;
	errs	+= irq_wait_test(fd, AIO168_WAIT_GSC_AO_BUF_THR_H2L, _high_to_low_ao_output);

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static int _burst_done(int fd)
{
	int	errs	= 0;
	int	i;
	int	ret;
	s32	val;

	val		= AIO168_IRQ2_AO_BURST_DONE;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_IRQ2_SEL, &val);
	errs	+= ret ? 1 : 0;

	val		= AIO168_AO_BURST_ENABLE_YES;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_AO_BURST_ENABLE, &val);
	errs	+= ret ? 1 : 0;

	val		= AIO168_AO_BURST_CLK_SRC_BCR;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_AO_BURST_CLK_SRC, &val);
	errs	+= ret ? 1 : 0;

	val		= 2500;	// 12,000 S/S
	ret		= aio168_ioctl(fd, AIO168_IOCTL_RBG_NRATE, &val);
	errs	+= ret ? 1 : 0;

	val		= AIO168_GEN_ENABLE_YES;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_RBG_ENABLE, &val);
	errs	+= ret ? 1 : 0;

	val		= AIO168_AO_CLK_SRC_RBG;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_AO_CLK_SRC, &val);
	errs	+= ret ? 1 : 0;

	val		= AIO168_AO_TIMING_SIMUL;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_AO_TIMING, &val);
	errs	+= ret ? 1 : 0;

	ret		= aio168_ioctl(fd, AIO168_IOCTL_AO_BUF_CLEAR, NULL);
	errs	+= ret ? 1 : 0;

	val		= 0;
	ret		= aio168_ioctl(fd, AIO168_IOCTL_AO_BUF_THR_LVL, &val);
	errs	+= ret ? 1 : 0;

	for (i = 0; i < 256; i++)
	{
		errs	+= aio168_reg_write(fd, -1, 0, AIO168_GSC_ODBR, 0x000000);
		errs	+= aio168_reg_write(fd, -1, 0, AIO168_GSC_ODBR, 0x010000);
		errs	+= aio168_reg_write(fd, -1, 0, AIO168_GSC_ODBR, 0x020000);
		errs	+= aio168_reg_write(fd, -1, 0, AIO168_GSC_ODBR, 0x1B0000);
	}

	// Notes: 1) >= 260 samples, 2) >= .25 sec delay, 3) sync pulse
	os_sleep_ms(300);
	ret		= aio168_ioctl(fd, AIO168_IOCTL_AO_SYNC, NULL);
	errs	+= ret ? 1 : 0;
	os_sleep_ms(10);
	return(errs);
}



//*****************************************************************************
static int _irq2_burst_end_test(int fd)
{
	int	errs	= 0;
	int	ret;

	gsc_label("Burst End");
	ret		= aio168_ioctl(fd, AIO168_IOCTL_INITIALIZE, NULL);
	errs	+= ret ? 1 : 0;
	errs	+= irq_wait_test(fd, AIO168_WAIT_GSC_AO_BURST_DONE, _burst_done);
	return(errs);
}



//*****************************************************************************
int irq2_test(int fd)
{
	int	errs	= 0;

	gsc_label("IRQ2");
	printf("\n");
	gsc_label_level_inc();

	errs	+= _irq2_ao_buf_thr_sts_l2h_test(fd);
	errs	+= _irq2_ao_buf_thr_sts_h2l_test(fd);
	errs	+= _irq2_burst_end_test(fd);

	gsc_label_level_dec();
	return(errs);
}



