// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO168/16aio168_Linux_2.x.x.x_DN/samples/rxrate/perform.c $
// $Rev: 53658 $
// $Date: 2023-08-31 15:55:05 -0500 (Thu, 31 Aug 2023) $

// 16AIO168: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1M		(1024 * 1024)



// variables ******************************************************************

static	u8	_data[_1M];



//*****************************************************************************
static int _configuration(const args_t* args)
{
	s32	ai_mode	= AIO168_AI_MODE_SINGLE;
	int	errs	= 0;
	s32	fsamp;
	s32	nrate	= 0xFFFF;

	errs	+= aio168_initialize			(args->fd, -1, 1);

	// Settings must be applied after the initialization call!

	errs	+= aio168_rx_io_mode			(args->fd, -1, 1, args->io_mode,				NULL);
	errs	+= aio168_rx_io_timeout			(args->fd, -1, 1, 30,							NULL);

	errs	+= aio168_data_format			(args->fd, -1, 1, AIO168_DATA_FORMAT_OFF_BIN,	NULL);
	errs	+= aio168_ai_mode				(args->fd, -1, 1, ai_mode,						NULL);
	errs	+= aio168_range					(args->fd, -1, 1, AIO168_RANGE_10V,				NULL);
	errs	+= aio168_ai_buf_thr_lvl		(args->fd, -1, 1, 16L * 1024,					NULL);
	errs	+= aio168_ai_scan_clk_src		(args->fd, -1, 1, AIO168_AI_SCAN_CLK_SRC_RAG,	NULL);
	errs	+= aio168_ai_scan_1_chan		(args->fd, -1, 1, 0,							NULL);

	errs	+= aio168_query					(args->fd, -1, 1, AIO168_QUERY_FSAMP_MAX, &fsamp);
	errs	+= aio168_ai_scan_size			(args->fd, -1, 1, args->scan_size,				NULL);

	switch (args->scan_size)
	{
		default:
		case AIO168_AI_SCAN_SIZE_SINGLE:					break;
		case AIO168_AI_SCAN_SIZE_0_1:		fsamp	/= 2;	break;
		case AIO168_AI_SCAN_SIZE_0_3:		fsamp	/= 4;	break;
		case AIO168_AI_SCAN_SIZE_0_7:		fsamp	/= 8;	break;
		case AIO168_AI_SCAN_SIZE_0_15:		fsamp	/= 16;	break;
	}

	errs	+= aio168_fsamp_ai_compute		(args->fd, -1, 1, fsamp, &nrate,				NULL);

	errs	+= aio168_rag_nrate				(args->fd, -1, 1, nrate,						NULL);
	errs	+= aio168_rag_enable			(args->fd, -1, 1, AIO168_GEN_ENABLE_YES,		NULL);
	errs	+= aio168_rbg_enable			(args->fd, -1, 1, AIO168_GEN_ENABLE_NO,			NULL);

	errs	+= aio168_fsamp_ai_report_all	(args->fd, -1, 1,								NULL);

	// Settings affecting the sample rate or voltage range must generally
	// be applied before autocalibration.
	// Refer to the board user manual for clarification.
	errs	+= aio168_autocal				(args->fd, -1, 1);	// Cable may need disconnecting for this.

	// Interrupt selections should be applied after autocalibration as the
	// driver overwrites the current selection to detect the Autocalibration
	// Done interrupt. If an application intends to detect the Autocalibration
	// Done interrupt, then the interrupt selection may be made beforehand.
	errs	+= aio168_irq0_sel				(args->fd, -1, 1, AIO168_IRQ0_IDLE_INIT,		NULL);
	errs	+= aio168_irq1_sel				(args->fd, -1, 1, AIO168_IRQ1_IDLE,				NULL);
	errs	+= aio168_irq2_sel				(args->fd, -1, 1, AIO168_IRQ2_IDLE,				NULL);

	// The final step is to clear the buffer.
	errs	+= aio168_ai_buf_clear			(args->fd, -1, 1);

	return(errs);
}



//*****************************************************************************
static int _rate_test(const args_t* args, long get)
{
	struct timeval	begin;
	int				errs	= 0;
	long			got;
	long long		limit	= (long long) args->rx_mb * 1000000L;
	struct timeval	minimum;
	struct timeval	now;
	long long		sps;
	long long		total	= 0;
	long			us;

	gsc_label("Reading");

	gettimeofday(&begin, NULL);
	minimum.tv_sec	= begin.tv_sec + args->seconds;
	minimum.tv_usec	= begin.tv_usec;

	for (;;)
	{
		got	= aio168_read(args->fd, _data, get);

		if (got < 0)
		{
			errs++;
			printf("FAIL <--- (read");
			printf(", total so far %lld", total);
			printf(", requested %ld bytes", (long) get);
			printf(", error %ld)\n", got);
			break;
		}

		gettimeofday(&now, NULL);
		total	+= got;

		if (total < limit)
			continue;

		if (now.tv_sec < minimum.tv_sec)
			continue;

		if ((now.tv_sec == minimum.tv_sec) && (now.tv_usec < minimum.tv_usec))
			continue;

		us	= (now.tv_sec - begin.tv_sec) * 1000000L
			+ (now.tv_usec - begin.tv_usec);
		sps	= (total * 10000000L / 4 / us + 5) / 10;

		printf(	"PASS  (");
		gsc_label_long_comma(total / 4);
		printf(" Samples, %ld.%06ld Seconds, ", us / 1000000, us % 1000000);
		gsc_label_long_comma(sps);
		printf(" S/S)\n");
		break;
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int		errs	= 0;
	long	get		= sizeof(_data);

	errs	+= _configuration(args);

	gsc_label ("Read Rate");
	printf("(%d second minimum, %ld MB minimum)\n", args->seconds, (long) args->rx_mb);
	gsc_label_level_inc();

	errs	+= aio168_ai_buf_clear(args->fd, -1, 1);

	errs	+= _rate_test(args, get);

	gsc_label_level_dec();
	return(errs);
}



