// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO168/16aio168_Linux_2.x.x.x_DN/utils/util_ai_mode.c $
// $Rev: 44231 $
// $Date: 2018-12-05 13:56:28 -0600 (Wed, 05 Dec 2018) $

// 16AIO168: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aio168_ai_mode
*
*	Purpose:
*
*		Provide a visual wrapper for the AIO168_IOCTL_AI_MODE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int aio168_ai_mode(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("AI Mode", index);

	ret		= aio168_ioctl(fd, AIO168_IOCTL_AI_MODE, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AIO168_AI_MODE_DIFF:

			ptr	= "Differential";
			break;

		case AIO168_AI_MODE_SINGLE:

			ptr	= "Single Ended";
			break;

		case AIO168_AI_MODE_ZERO:

			ptr	= "Zero Voltage Test";
			break;

		case AIO168_AI_MODE_VREF:

			ptr	= "Vref Test";
			break;

		case AIO168_AI_MODE_AO_0:

			ptr	= "Loopback Output Channel 0";
			break;

		case AIO168_AI_MODE_AO_1:

			ptr	= "Loopback Output Channel 1";
			break;

		case AIO168_AI_MODE_AO_2:

			ptr	= "Loopback Output Channel 2";
			break;

		case AIO168_AI_MODE_AO_3:

			ptr	= "Loopback Output Channel 3";
			break;

		case AIO168_AI_MODE_AO_4:

			ptr	= "Loopback Output Channel 4";
			break;

		case AIO168_AI_MODE_AO_5:

			ptr	= "Loopback Output Channel 5";
			break;

		case AIO168_AI_MODE_AO_6:

			ptr	= "Loopback Output Channel 6";
			break;

		case AIO168_AI_MODE_AO_7:

			ptr	= "Loopback Output Channel 7";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


