// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO168/16aio168_Linux_2.x.x.x_DN/utils/util_ao_burst_clk_src.c $
// $Rev: 44231 $
// $Date: 2018-12-05 13:56:28 -0600 (Wed, 05 Dec 2018) $

// 16AIO168: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aio168_ao_burst_clk_src
*
*	Purpose:
*
*		Provide a visual wrapper for the AIO168_IOCTL_AO_BURST_CLK_SRC service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int aio168_ao_burst_clk_src(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("AO Burst Clock Source", index);

	ret		= aio168_ioctl(fd, AIO168_IOCTL_AO_BURST_CLK_SRC, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AIO168_AO_BURST_CLK_SRC_RAG:

			ptr	= "Rate-A Generator";
			break;

		case AIO168_AO_BURST_CLK_SRC_RBG:

			ptr	= "Rate-B Generator";
			break;

		case AIO168_AO_BURST_CLK_SRC_EXT:

			ptr	= "External Clock Input";
			break;

		case AIO168_AO_BURST_CLK_SRC_BCR:

			ptr	= "BCR AO Sync";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


