// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO168/16aio168_Linux_2.x.x.x_DN/utils/util_auto_cal_sts.c $
// $Rev: 53661 $
// $Date: 2023-08-31 15:59:26 -0500 (Thu, 31 Aug 2023) $

// 16AIO168: Utilities: source file

#include "main.h"



// Retained for backwards compatibility only:

/******************************************************************************
*
*	Function:	aio168_auto_cal_sts
*
*	Purpose:
*
*		Provide a visual wrapper for the AIO168_IOCTL_AUTO_CAL_STS service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int aio168_auto_cal_sts(int fd, int index, int verbose, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;
	s32			set;

	if (verbose)
		gsc_label_index("Autocalibration Status", index);

	ret		= aio168_ioctl(fd, AIO168_IOCTL_AUTO_CAL_STS, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AIO168_AUTO_CAL_STS_ACTIVE:

			ptr	= "Active";
			break;

		case AIO168_AUTO_CAL_STS_FAIL:

			ptr	= "Fail";
			break;

		case AIO168_AUTO_CAL_STS_PASS:

			ptr	= "Pass";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


