// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO168/16aio168_Linux_2.x.x.x_DN/utils/util_config_ao.c $
// $Rev: 53661 $
// $Date: 2023-08-31 15:59:26 -0500 (Thu, 31 Aug 2023) $

// 16AIO168: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aio168_config_ao
*
*	Purpose:
*
*		Configure the given device's analog output options.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	This is the desired Fsamp rate, of -1 to use the default.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aio168_config_ao(int fd, int index, int verbose, s32 fsamp)
{
	int	errs	= 0;
	s32	nrate	= 0xFFFF;

	if (verbose)
	{
		gsc_label_index("Output Configuration", index);
		printf("\n");
		gsc_label_level_inc();
	}

	errs	+= aio168_initialize			(fd, index, verbose);

	// Settings must be applied after the initialization call!

	errs	+= aio168_tx_io_mode			(fd, index, verbose, GSC_IO_MODE_PIO,				NULL);
	errs	+= aio168_tx_io_timeout			(fd, index, verbose, 30,							NULL);

	errs	+= aio168_data_format			(fd, index, verbose, AIO168_DATA_FORMAT_OFF_BIN,	NULL);

	errs	+= aio168_ao_buf_thr_lvl		(fd, index, verbose, 16L * 1024,					NULL);
	errs	+= aio168_ao_burst_enable		(fd, index, verbose, AIO168_AO_BURST_ENABLE_NO,		NULL);
	errs	+= aio168_ao_burst_clk_src		(fd, index, verbose, AIO168_AO_BURST_CLK_SRC_BCR,	NULL);
	errs	+= aio168_ao_looping			(fd, index, verbose, AIO168_AO_LOOPING_DISABLE,		NULL);
	errs	+= aio168_ao_timing				(fd, index, verbose, AIO168_AO_TIMING_SIMUL,		NULL);
	errs	+= aio168_range					(fd, index, verbose, AIO168_RANGE_10V,				NULL);
	errs	+= aio168_fsamp_ao_compute		(fd, index, verbose, fsamp, &nrate,					NULL);
	errs	+= aio168_ao_clk_src			(fd, index, verbose, AIO168_AO_CLK_SRC_RBG,			NULL);
	errs	+= aio168_rbg_clk_src			(fd, index, verbose, AIO168_RBG_CLK_SRC_MASTER,		NULL);
	errs	+= aio168_rbg_enable			(fd, index, verbose, AIO168_GEN_ENABLE_YES,			NULL);
	errs	+= aio168_rbg_nrate				(fd, index, verbose, nrate,							NULL);

	errs	+= aio168_fsamp_ao_report_all	(fd, index, verbose,								NULL);

	// Settings affecting the sample rate or voltage range must generally
	// be applied before autocalibration.
	// Refer to the board user manual for clarification.
	errs	+= aio168_autocal				(fd, index, verbose); // cable must be removed

	// Interrupt selections should be applied after autocalibration as the
	// driver overwrites the current selection to detect the Autocalibration
	// Done interrupt. If an application intends to detect the Autocalibration
	// Done interrupt, then the interrupt selection may be made beforehand.
	errs	+= aio168_irq0_sel				(fd, index, verbose, AIO168_IRQ0_IDLE_INIT,			NULL);
	errs	+= aio168_irq1_sel				(fd, index, verbose, AIO168_IRQ1_IDLE,				NULL);
	errs	+= aio168_irq2_sel				(fd, index, verbose, AIO168_IRQ2_IDLE,				NULL);

	// The final step is to clear the buffer.
	errs	+= aio168_ao_buf_clear			(fd, index, verbose);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}


