// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO168/16aio168_Linux_2.x.x.x_DN/utils/util_fsamp_ai.c $
// $Rev: 53661 $
// $Date: 2023-08-31 15:59:26 -0500 (Thu, 31 Aug 2023) $

// 16AIO168: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	ABS(v)					(((v) < 0) ? (-(v)) : (v))
#define	LIMIT_MAX(v,max)		(((v) > (max)) ? (max) : (v))
#define	LIMIT_MIN(v,min)		(((v) < (min)) ? (min) : (v))
#define	LIMIT_RANGE(v,min,max)	(LIMIT_MIN((LIMIT_MAX((v),(max))),(min)))



//*****************************************************************************
static int _aio168_fsamp_ai_get_mode_rag(int fd, const char* test, int chan, s32* get)
{
	s32	enable;
	int	errs		= 0;
	s32	master;
	s32	nrate;
	s32	set			= 0;

	test	= test ? test : "";

	errs	+= aio168_query(fd, -1, 0, AIO168_QUERY_FREF_DEFAULT, &master);
	errs	+= aio168_rag_enable(fd, -1, 0, -1, &enable);
	errs	+= aio168_rag_nrate(fd, -1, 0, -1, &nrate);

	if (errs)
	{
	}
	else if (enable == AIO168_GEN_ENABLE_NO)
	{
		gsc_label_long_comma(set);
		printf(" S/S  (%sChan %d, Ref ", test, chan);
		gsc_label_long_comma(master);
		printf(" Hz, Gen A Disabled)\n");
	}
	else
	{
		set	= master / nrate;
		gsc_label_long_comma(set);
		printf(" S/S  (%sChan %d, Ref ", test, chan);
		gsc_label_long_comma(master);
		printf(" Hz, Gen A, Nrate ");
		gsc_label_long_comma(nrate);
		printf(")\n");
	}

	if (get)
		get[0]	= set;

	return(errs);
}



//*****************************************************************************
static int _aio168_fsamp_ai_get_mode_rbg(int fd, const char* test, int chan, s32* get)
{
	s32	a_enable;
	s32	a_nrate;
	s32	b_enable;
	s32	b_nrate;
	s32	b_src;
	int	errs		= 0;
	s32	master;
	s32	set			= 0;

	test	= test ? test : "";

	errs	+= aio168_query(fd, -1, 0, AIO168_QUERY_FREF_DEFAULT, &master);
	errs	+= aio168_rag_enable(fd, -1, 0, -1, &a_enable);
	errs	+= aio168_rag_nrate(fd, -1, 0, -1, &a_nrate);
	errs	+= aio168_rbg_enable(fd, -1, 0, -1, &b_enable);
	errs	+= aio168_rbg_nrate(fd, -1, 0, -1, &b_nrate);
	errs	+= aio168_rbg_clk_src(fd, -1, 0, -1, &b_src);

	if (errs)
	{
	}
	else if (b_src == AIO168_RBG_CLK_SRC_MASTER)
	{
		if (b_enable == AIO168_GEN_ENABLE_NO)
		{
			gsc_label_long_comma(set);
			printf(" S/S  (%sChan %d, Ref ", test, chan);
			gsc_label_long_comma(master);
			printf(" Hz, Gen B Disabled)\n");
		}
		else
		{
			set	= master / b_nrate;
			gsc_label_long_comma(set);
			printf(" S/S  (%sChan %d, Ref ", test, chan);
			gsc_label_long_comma(master);
			printf(" Hz, Gen B, Nrate ");
			gsc_label_long_comma(b_nrate);
			printf(")\n");
		}
	}
	else if (a_enable == AIO168_GEN_ENABLE_NO)
	{
		gsc_label_long_comma(set);
		printf(" S/S  (%sChan %d, Ref ", test, chan);
		gsc_label_long_comma(master);
		printf(" Hz, Gen A Disabled)\n");
	}
	else
	{
		set	= master / (a_nrate * b_nrate);
		gsc_label_long_comma(set);
		printf(" S/S  (%sChan %d, Ref ", test, chan);
		gsc_label_long_comma(master);
		printf(" Hz, Gen A, Nrate ");
		gsc_label_long_comma(a_nrate);
		printf(", Gen B, Nrate ");
		gsc_label_long_comma(b_nrate);
		printf(")\n");
	}

	if (get)
		get[0]	= set;

	return(errs);
}



//*****************************************************************************
static int _aio168_fsamp_ai_get_mode_ext(int fd, const char* test, int chan, s32* get)
{
	s32	set		= 0;

	test	= test ? test : "";
	gsc_label_long_comma(set);
	printf(" S/S  (%sChan %d, External Clock)\n", test, chan);

	if (get)
		get[0]	= set;

	return(0);
}



//*****************************************************************************
static int _aio168_fsamp_ai_get_mode_bcr(int fd, const char* test, int chan, s32* get)
{
	s32	set		= 0;

	test	= test ? test : "";
	gsc_label_long_comma(set);
	printf(" S/S  (%sChan %d, Software Sync)\n", test, chan);

	if (get)
		get[0]	= set;

	return(0);
}



//*****************************************************************************
static int _aio168_fsamp_ai_get_mode(int fd, int chan, s32* get)
{
	int	errs;
	s32	set			= 0;
	s32	src;

	errs	= aio168_ai_scan_clk_src(fd, -1, 0, -1, &src);

	if (errs == 0)
	{
		switch (src)
		{
			default:

				errs++;
				printf(	"FAIL <---  (%d. unrecognized source: %ld)\n",
						__LINE__,
						(long) src);
				break;

			case AIO168_AI_SCAN_CLK_SRC_RAG:	_aio168_fsamp_ai_get_mode_rag(fd, "", chan, &set);	break;
			case AIO168_AI_SCAN_CLK_SRC_RBG:	_aio168_fsamp_ai_get_mode_rbg(fd, "", chan, &set);	break;
			case AIO168_AI_SCAN_CLK_SRC_EXT:	_aio168_fsamp_ai_get_mode_ext(fd, "", chan, &set);	break;
			case AIO168_AI_SCAN_CLK_SRC_BCR:	_aio168_fsamp_ai_get_mode_bcr(fd, "", chan, &set);	break;
		}
	}

	if (get)
		get[0]	= set;

	return(errs);
}



//*****************************************************************************
static int _aio168_fsamp_ai_get_diff(int fd, int chan, s32* get)
{
	s32	ch;
	int	errs	= 0;
	s32	max		= 0;
	s32	min		= 0;
	s32	set		= 0;
	s32	size;

	errs	+= aio168_ai_scan_size(fd, -1, 0, -1, &size);
	errs	+= aio168_ai_scan_1_chan(fd, -1, 0, -1, &ch);

	if (errs == 0)
	{
		switch (size)
		{
			default:

				errs++;
				printf(	"FAIL <---  (%d. unrecognized size: %ld)\n",
						__LINE__,
						(long) size);
				break;

			case AIO168_AI_SCAN_SIZE_SINGLE:	min	= ch;	max	= ch;	break;
			case AIO168_AI_SCAN_SIZE_0_1:		min	= 0;	max	= 1;	break;
			case AIO168_AI_SCAN_SIZE_0_3:		min	= 0;	max	= 3;	break;
			case AIO168_AI_SCAN_SIZE_0_7:		min	= 0;	max	= 7;	break;
			case AIO168_AI_SCAN_SIZE_0_15:		min	= 0;	max	= 15;	break;
		}

		if (errs)
		{
		}
		else if ((chan >= min) && (chan <= max))
		{
			errs	+= _aio168_fsamp_ai_get_mode(fd, chan, &set);
		}
		else
		{
			gsc_label_long_comma(set);
			printf(" S/S  (Chan %d, invalid channel)\n", chan);
		}
	}

	if (get)
		get[0]	= set;

	return(errs);
}



//*****************************************************************************
static int _aio168_fsamp_ai_get_single(int fd, int chan, s32* get)
{
	int	errs;
	s32	set		= 0;

	// Single Ended configuration
	errs	= _aio168_fsamp_ai_get_mode(fd, chan, &set);

	if (get)
		get[0]	= set;

	return(errs);
}



//*****************************************************************************
static int _aio168_fsamp_ai_get_test(int fd, const char* name, int chan, s32* get)
{
	char	buf[128];
	int		errs;
	s32		set			= 0;
	s32		src;

	errs	= aio168_ai_scan_clk_src(fd, -1, 0, -1, &src);

	if (errs)
	{
	}
	else if (chan == 0)
	{
		sprintf(buf, "%s Test Mode, ", name);

		switch (src)
		{
			default:

				errs++;
				printf(	"FAIL <---  (%d. unrecognized source: %ld)\n",
						__LINE__,
						(long) src);
				break;

			case AIO168_AI_SCAN_CLK_SRC_RAG:	errs	= _aio168_fsamp_ai_get_mode_rag(fd, buf, chan, get);	break;
			case AIO168_AI_SCAN_CLK_SRC_RBG:	errs	= _aio168_fsamp_ai_get_mode_rbg(fd, buf, chan, get);	break;
			case AIO168_AI_SCAN_CLK_SRC_EXT:	errs	= _aio168_fsamp_ai_get_mode_ext(fd, buf, chan, get);	break;
			case AIO168_AI_SCAN_CLK_SRC_BCR:	errs	= _aio168_fsamp_ai_get_mode_bcr(fd, buf, chan, get);	break;
		}
	}
	else
	{
		gsc_label_long_comma(set);
		printf(" S/S  (%s Test Mode, Chan %d, invalid channel)\n", name, chan);
	}

	if (get)
		get[0]	= set;

	return(errs);
}



//*****************************************************************************
static int _aio168_fsamp_ai_get_zero(int fd, int chan, s32* get)
{
	int	errs;

	errs	= _aio168_fsamp_ai_get_test(fd, "Zero Volt", chan, get);
	return(errs);
}



//*****************************************************************************
static int _aio168_fsamp_ai_get_vref(int fd, int chan, s32* get)
{
	int	errs;

	errs	= _aio168_fsamp_ai_get_test(fd, "Vref", chan, get);
	return(errs);
}



//*****************************************************************************
static int _aio168_fsamp_ai_get_ao_0(int fd, int chan, s32* get)
{
	int	errs;

	errs	= _aio168_fsamp_ai_get_test(fd, "AO Chan 0", chan, get);
	return(errs);
}



//*****************************************************************************
static int _aio168_fsamp_ai_get_ao_1(int fd, int chan, s32* get)
{
	int	errs;

	errs	= _aio168_fsamp_ai_get_test(fd, "AO Chan 1", chan, get);
	return(errs);
}



//*****************************************************************************
static int _aio168_fsamp_ai_get_ao_2(int fd, int chan, s32* get)
{
	int	errs;

	errs	= _aio168_fsamp_ai_get_test(fd, "AO Chan 2", chan, get);
	return(errs);
}



//*****************************************************************************
static int _aio168_fsamp_ai_get_ao_3(int fd, int chan, s32* get)
{
	int	errs;

	errs	= _aio168_fsamp_ai_get_test(fd, "AO Chan 3", chan, get);
	return(errs);
}



//*****************************************************************************
static int _aio168_fsamp_ai_get_ao_4(int fd, int chan, s32* get)
{
	int	errs;

	errs	= _aio168_fsamp_ai_get_test(fd, "AO Chan 4", chan, get);
	return(errs);
}



//*****************************************************************************
static int _aio168_fsamp_ai_get_ao_5(int fd, int chan, s32* get)
{
	int	errs;

	errs	= _aio168_fsamp_ai_get_test(fd, "AO Chan 5", chan, get);
	return(errs);
}



//*****************************************************************************
static int _aio168_fsamp_ai_get_ao_6(int fd, int chan, s32* get)
{
	int	errs;

	errs	= _aio168_fsamp_ai_get_test(fd, "AO Chan 6", chan, get);
	return(errs);
}



//*****************************************************************************
static int _aio168_fsamp_ai_get_ao_7(int fd, int chan, s32* get)
{
	int	errs;

	errs	= _aio168_fsamp_ai_get_test(fd, "AO Chan 7", chan, get);
	return(errs);
}



/******************************************************************************
*
*	Function:	aio168_fsamp_ai_compute
*
*	Purpose:
*
*		Determine the best configuration for the specified input sample rate.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	The desired sample rate.
*
*		nrate	The Nrate value is put here. MUST NOT BE NULL!
*
*		rate	Report the sample rate achieved here, if non-NULL.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aio168_fsamp_ai_compute(int fd, int index, int verbose, s32 fsamp, s32* nrate, double* rate)
{
	double	delta;
	double	delta_best	= FLT_MAX;
	int		errs		= 0;
	double	fsamp_best	= 0;
	double	fsamp_got;
	s32		fsamp_max;	// Queried value.
	s32		fsamp_min;	// Queried value.
	double	fsamp_want;
	int		init		= 0;	// Have we performed the first run initialization?
	s32		master;
	s32		nr;
	s32		nr_max;
	s32		nr_min;
	s32		nr_use;
	s32		nrate_best	= 0;
	s32		nrate_max;	// Queried value.
	s32		nrate_min;	// Queried value.

	if (verbose)
	{
		gsc_label_index("AI Sample Rate", index);
		gsc_label_long_comma(fsamp);
		printf(" S/S");
	}

	errs	+= aio168_query(fd, -1, 0, AIO168_QUERY_FSAMP_MAX, &fsamp_max);
	errs	+= aio168_query(fd, -1, 0, AIO168_QUERY_FSAMP_MIN, &fsamp_min);
	errs	+= aio168_query(fd, -1, 0, AIO168_QUERY_FREF_DEFAULT, &master);
	errs	+= aio168_query(fd, -1, 0, AIO168_QUERY_NRATE_MAX, &nrate_max);
	errs	+= aio168_query(fd, -1, 0, AIO168_QUERY_NRATE_MIN, &nrate_min);

	fsamp_want	= (double) LIMIT_RANGE(fsamp, fsamp_min, fsamp_max);
	nr			= master / fsamp_want;
	nr_min		= nr - 1;
	nr_max		= nr + 1;
	nr_min		= (nr_min < nrate_min) ? nrate_min : nr_min;
	nr_max		= (nr_max > nrate_max) ? nrate_max : nr_max;

	for (nr = nr_min; nr <= nr_max; nr++)
	{
		if (nr < nr_min)
			nr_use	= nr_min;
		else if (nr > nr_max)
			nr_use	= nr_max;
		else
			nr_use	= nr;

		fsamp_got	= ((double) master) / nr_use;
		delta		= fsamp_got - fsamp_want;

		if ((init == 0) || ((ABS(delta)) < (ABS(delta_best))))
		{
			init		= 1;
			delta_best	= delta;
			fsamp_best	= fsamp_got;
			nrate_best	= nr_use;
		}
	}

	nrate[0]	= nrate_best;

	if (rate)
		rate[0]		= fsamp_best;

	if (verbose)
	{
		printf("%s  (", errs ? " FAIL <---" : "");
		printf("Fref ");
		gsc_label_long_comma((long) master);
		printf(", Nrate ");
		gsc_label_long_comma((long) nrate_best);
		printf(", Fsamp %.3f", fsamp_best);
		printf(")\n");
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	aio168_fsamp_ai_get
*
*	Purpose:
*
*		Determine the input sample rate.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		title	Show a title?
*
*		chan	The channel for which the request is being made.
*
*		use		Is this channel being used?
*
*		get		Store the sample rate here, if non-NULL.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aio168_fsamp_ai_get(int fd, int chan, int title, int use, s32* get)
{
	int	errs;
	s32	mode;
	s32	set			= 0;

	if (title)
		gsc_label("AI Sample Rate");

	errs	= aio168_ai_mode(fd, -1, 0, -1, &mode);

	if (errs)
	{
	}
	else if (use == 0)
	{
		set		= 0;
		gsc_label_long_comma(set);
		printf(" S/S  (Channel Not Used)\n");
	}
	else
	{
		switch (mode)
		{
			default:

				errs++;
				printf(	"FAIL <---  (%d. unrecognized mode: %ld)\n",
						__LINE__,
						(long) mode);
				break;

			case AIO168_AI_MODE_DIFF:	errs	+= _aio168_fsamp_ai_get_diff(fd, chan, &set);	break;
			case AIO168_AI_MODE_SINGLE:	errs	+= _aio168_fsamp_ai_get_single(fd, chan, &set);	break;
			case AIO168_AI_MODE_ZERO:	errs	+= _aio168_fsamp_ai_get_zero(fd, chan, &set);	break;
			case AIO168_AI_MODE_VREF:	errs	+= _aio168_fsamp_ai_get_vref(fd, chan, &set);	break;
			case AIO168_AI_MODE_AO_0:	errs	+= _aio168_fsamp_ai_get_ao_0(fd, chan, &set);	break;
			case AIO168_AI_MODE_AO_1:	errs	+= _aio168_fsamp_ai_get_ao_1(fd, chan, &set);	break;
			case AIO168_AI_MODE_AO_2:	errs	+= _aio168_fsamp_ai_get_ao_2(fd, chan, &set);	break;
			case AIO168_AI_MODE_AO_3:	errs	+= _aio168_fsamp_ai_get_ao_3(fd, chan, &set);	break;
			case AIO168_AI_MODE_AO_4:	errs	+= _aio168_fsamp_ai_get_ao_4(fd, chan, &set);	break;
			case AIO168_AI_MODE_AO_5:	errs	+= _aio168_fsamp_ai_get_ao_5(fd, chan, &set);	break;
			case AIO168_AI_MODE_AO_6:	errs	+= _aio168_fsamp_ai_get_ao_6(fd, chan, &set);	break;
			case AIO168_AI_MODE_AO_7:	errs	+= _aio168_fsamp_ai_get_ao_7(fd, chan, &set);	break;
		}
	}


	if (get)
		get[0]	= set;

	return(errs);
}



/******************************************************************************
*
*	Function:	aio168_fsamp_ai_report
*
*	Purpose:
*
*		Determine and report the input sample rate for the specified channel.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		chan	The index of the channel of interest.
*
*		use		Use this channel?
*
*		get		Store the sample rate here, if non-NULL.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aio168_fsamp_ai_report(int fd, int index, int chan, int use, s32* get)
{
	char	buf[64];
	int		errs;

	sprintf(buf, "AI Channel %d", chan);
	gsc_label_index(buf, index);
	errs	= aio168_fsamp_ai_get(fd, chan, 0, use, get);
	return(errs);
}



/******************************************************************************
*
*	Function:	aio168_fsamp_ai_report_all
*
*	Purpose:
*
*		Determine and report the input sample rate for all channels.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	Return th eoverall rate here.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aio168_fsamp_ai_report_all(int fd, int index, int verbose, s32* fsamp)
{
	s32	chan;
	int	errs	= 0;
	int	i;
	u16	mask;
	int	max;
	int	min;
	s32	mode;
	s32	size;
	s32	sps;
	s32	total	= 0;
	int	use;

	if (verbose)
		gsc_label_index("AI Sample Rates", index);

	errs	+= aio168_ai_mode(fd, -1, 0, -1, &mode);
	errs	+= aio168_ai_scan_size(fd, -1, 0, -1, &size);
	errs	+= aio168_ai_scan_1_chan(fd, -1, 0, -1, &chan);

	if (errs)
	{
	}
	else
	{
		if (verbose)
		{
			printf("\n");
			gsc_label_level_inc();
		}

		switch (size)
		{
			default:
			case AIO168_AI_SCAN_SIZE_SINGLE:	min	= chan;	max	= chan;	break;
			case AIO168_AI_SCAN_SIZE_0_1:		min	= 0;	max	= 1;	break;
			case AIO168_AI_SCAN_SIZE_0_3:		min	= 0;	max	= 3;	break;
			case AIO168_AI_SCAN_SIZE_0_7:		min	= 0;	max	= 7;	break;
			case AIO168_AI_SCAN_SIZE_0_15:		min	= 0;	max	= 15;	break;
		}

		switch (mode)
		{
			case AIO168_AI_MODE_DIFF:

				mask	= 0x5555;
				break;

			default:
			case AIO168_AI_MODE_SINGLE:
			case AIO168_AI_MODE_ZERO:
			case AIO168_AI_MODE_VREF:
			case AIO168_AI_MODE_AO_0:
			case AIO168_AI_MODE_AO_1:
			case AIO168_AI_MODE_AO_2:
			case AIO168_AI_MODE_AO_3:
			case AIO168_AI_MODE_AO_4:
			case AIO168_AI_MODE_AO_5:
			case AIO168_AI_MODE_AO_6:
			case AIO168_AI_MODE_AO_7:

				mask	= 0xFFFF;
				break;
		}

		for (i = min; i <= max; i++)
		{
			use		= ((0x1 << i) & mask) ? 1 : 0;
			sps		= 0;
			errs	+= aio168_fsamp_ai_report(fd, -1, i, use, &sps);

			total	+= sps;
		}

		if (verbose)
		{
			gsc_label("Overall AI Rate");
			gsc_label_long_comma(total);
			printf(" S/S\n");
			gsc_label_level_dec();
		}
	}

	if (fsamp)
		fsamp[0]	= total;

	return(errs);
}


