// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO168/16aio168_Linux_2.x.x.x_DN/utils/util_fsamp_ao.c $
// $Rev: 51163 $
// $Date: 2022-06-28 14:13:36 -0500 (Tue, 28 Jun 2022) $

// 16AIO168: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	ABS(v)					(((v) < 0) ? (-(v)) : (v))
#define	LIMIT_MAX(v,max)		(((v) > (max)) ? (max) : (v))
#define	LIMIT_MIN(v,min)		(((v) < (min)) ? (min) : (v))
#define	LIMIT_RANGE(v,min,max)	(LIMIT_MIN((LIMIT_MAX((v),(max))),(min)))



//*****************************************************************************
static int _aio168_fsamp_ao_get_rag(int fd, s32* get)
{
	s32	enable;
	int	errs		= 0;
	s32	master;
	s32	nrate;
	s32	set			= 0;

	errs	+= aio168_query(fd, -1, 0, AIO168_QUERY_FREF_DEFAULT, &master);
	errs	+= aio168_rag_enable(fd, -1, 0, -1, &enable);
	errs	+= aio168_rag_nrate(fd, -1, 0, -1, &nrate);

	if (errs)
	{
	}
	else if (enable == AIO168_GEN_ENABLE_NO)
	{
		gsc_label_long_comma(set);
		printf(" S/S  (Ref ");
		gsc_label_long_comma(master);
		printf(" Hz, Gen A Disabled)\n");
	}
	else
	{
		set	= master / nrate;
		gsc_label_long_comma(set);
		printf(" S/S  (Ref ");
		gsc_label_long_comma(master);
		printf(" Hz, Gen A, Nrate ");
		gsc_label_long_comma(nrate);
		printf(")\n");
	}

	if (get)
		get[0]	= set;

	return(errs);
}



//*****************************************************************************
static int _aio168_fsamp_ao_get_rbg(int fd, s32* get)
{
	s32	a_enable;
	s32	a_nrate;
	s32	b_enable;
	s32	b_nrate;
	s32	b_src;
	int	errs		= 0;
	s32	master;
	s32	set			= 0;

	errs	+= aio168_query(fd, -1, 0, AIO168_QUERY_FREF_DEFAULT, &master);
	errs	+= aio168_rag_enable(fd, -1, 0, -1, &a_enable);
	errs	+= aio168_rag_nrate(fd, -1, 0, -1, &a_nrate);
	errs	+= aio168_rbg_enable(fd, -1, 0, -1, &b_enable);
	errs	+= aio168_rbg_nrate(fd, -1, 0, -1, &b_nrate);
	errs	+= aio168_rbg_clk_src(fd, -1, 0, -1, &b_src);

	if (errs)
	{
	}
	else if (b_src == AIO168_RBG_CLK_SRC_MASTER)
	{
		if (b_enable == AIO168_GEN_ENABLE_NO)
		{
			gsc_label_long_comma(set);
			printf(" S/S  (Ref ");
			gsc_label_long_comma(master);
			printf(" Hz, Gen B Disabled)\n");
		}
		else
		{
			set	= master / b_nrate;
			gsc_label_long_comma(set);
			printf(" S/S  (Ref ");
			gsc_label_long_comma(master);
			printf(" Hz, Gen B, Nrate ");
			gsc_label_long_comma(b_nrate);
			printf(")\n");
		}
	}
	else if (a_enable == AIO168_GEN_ENABLE_NO)
	{
		gsc_label_long_comma(set);
		printf(" S/S  (Ref ");
		gsc_label_long_comma(master);
		printf(" Hz, Gen A Disabled)\n");
	}
	else
	{
		set	= master / (a_nrate * b_nrate);
		gsc_label_long_comma(set);
		printf(" S/S  (Ref ");
		gsc_label_long_comma(master);
		printf(" Hz, Gen A, Nrate ");
		gsc_label_long_comma(a_nrate);
		printf(", Gen B, Nrate ");
		gsc_label_long_comma(b_nrate);
		printf(")\n");
	}

	if (get)
		get[0]	= set;

	return(errs);
}



//*****************************************************************************
static int _aio168_fsamp_ao_get_ext(int fd, s32* get)
{
	s32	set		= 0;

	gsc_label_long_comma(set);
	printf(" S/S  (External Clock)\n");

	if (get)
		get[0]	= set;

	return(0);
}



//*****************************************************************************
static int _aio168_fsamp_ao_get_disable(int fd, s32* get)
{
	s32	set		= 0;

	gsc_label_long_comma(set);
	printf(" S/S  (Channel Disabled)\n");

	if (get)
		get[0]	= set;

	return(0);
}



//*****************************************************************************
static int _aio168_fsamp_ao_get(int fd, s32* get, int use)
{
	int	errs		= 0;
	s32	set			= 0;
	s32	src;

	if (use)
	{
		errs	+= aio168_ao_clk_src(fd, -1, 0, -1, &src);

		if (errs == 0)
		{
			switch (src)
			{
				default:

					errs++;
					printf(	"FAIL <---  (%d. unrecognized source: %ld)\n",
							__LINE__,
							(long) src);
					break;

				case AIO168_AO_CLK_SRC_RAG:		_aio168_fsamp_ao_get_rag(fd, &set);		break;
				case AIO168_AO_CLK_SRC_RBG:		_aio168_fsamp_ao_get_rbg(fd, &set);		break;
				case AIO168_AO_CLK_SRC_EXT:		_aio168_fsamp_ao_get_ext(fd, &set);		break;
				case AIO168_AO_CLK_SRC_DISABLE:	_aio168_fsamp_ao_get_disable(fd, &set);	break;
			}
		}
	}
	else
	{
		set		= 0;
		gsc_label_long_comma(set);
		printf(" S/S  (Channel Not Used)\n");
	}

	if (get)
		get[0]	= set;

	return(errs);
}



/******************************************************************************
*
*	Function:	aio168_fsamp_ao_compute
*
*	Purpose:
*
*		Determine the best configuration for the specified output sample rate.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	The desired sample rate.
*
*		nrate	The Nrate value is put here. MUST NOT BE NULL!
*
*		rate	Report the sample rate achieved here, if non-NULL.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aio168_fsamp_ao_compute(int fd, int index, int verbose, s32 fsamp, s32* nrate, double* rate)
{
	double	delta;
	double	delta_best	= FLT_MAX;
	int		errs		= 0;
	s32		fref;
	double	fsamp_best	= 0;
	double	fsamp_got;
	s32		fsamp_max;	// Queried value.
	s32		fsamp_min;	// Queried value.
	double	fsamp_want;
	s32		nrate_best	= 0;
	s32		nrate_ll;	// lower limit
	s32		nrate_max;	// Queried value.
	s32		nrate_min;	// Queried value.
	s32		nrate_ul;	// upper limit
	s32		nrate_use;

	if (verbose)
	{
		gsc_label_index("AO Sample Rate", index);
		gsc_label_long_comma(fsamp);
		printf(" S/S");
	}

	errs	+= aio168_query(fd, -1, 0, AIO168_QUERY_FREF_DEFAULT, &fref);
	errs	+= aio168_query(fd, -1, 0, AIO168_QUERY_FSAMP_MAX, &fsamp_max);
	errs	+= aio168_query(fd, -1, 0, AIO168_QUERY_FSAMP_MIN, &fsamp_min);
	errs	+= aio168_query(fd, -1, 0, AIO168_QUERY_NRATE_MAX, &nrate_max);
	errs	+= aio168_query(fd, -1, 0, AIO168_QUERY_NRATE_MIN, &nrate_min);

	fsamp_want	= (double) LIMIT_RANGE(fsamp, fsamp_min, fsamp_max);
	nrate_use	= fref / fsamp_want;
	nrate_ll	= nrate_use - 1;
	nrate_ul	= nrate_use + 1;
	nrate_ll	= (nrate_ll < nrate_min) ? nrate_min : nrate_ll;
	nrate_ul	= (nrate_ul > nrate_max) ? nrate_max : nrate_ul;

	for (nrate_use = nrate_ll; nrate_use <= nrate_ul; nrate_use++)
	{
		fsamp_got	= ((double) fref) / nrate_use;
		delta		= fsamp_got - fsamp_want;

		if ((ABS(delta)) < (ABS(delta_best)))
		{
			delta_best	= delta;
			fsamp_best	= fsamp_got;
			nrate_best	= nrate_use;
		}
	}

	nrate[0]	= nrate_best;

	if (rate)
		rate[0]	= fsamp_best;

	if (verbose)
	{
		printf("%s  (", errs ? " FAIL <---" : "");
		printf("Fref ");
		gsc_label_long_comma(fref);
		printf(", Nrate ");
		gsc_label_long_comma(nrate_best);
		printf(", Fsamp %.3f", fsamp_best);
		printf(")\n");
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	aio168_fsamp_ao_report
*
*	Purpose:
*
*		Determine and report the output sample rate for the specified channel.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		chan	The index of the channel of interest.
*
*		fsamp	Store the sample rate here, if non-NULL.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aio168_fsamp_ao_report(int fd, int index, int verbose, int chan, s32* fsamp)
{
	char	buf[64];
	int		errs;

	if (verbose)
	{
		sprintf(buf, "AO Channel %d", chan);
		gsc_label_index(buf, index);
	}

	errs	= _aio168_fsamp_ao_get(fd, fsamp, 1);
	return(errs);
}



/******************************************************************************
*
*	Function:	aio168_fsamp_ao_report_all
*
*	Purpose:
*
*		Determine and report the output sample rate for all channels.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	Report the sampe rate here.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aio168_fsamp_ao_report_all(int fd, int index, int verbose, s32* fsamp)
{
	int	errs	= 0;
	int	i;
	s32	sps;
	s32	total	= 0;

	if (verbose)
	{
		gsc_label_index("AO Sample Rates", index);
		printf("\n");
		gsc_label_level_inc();
	}

	for (i = 0; i <= 7; i++)
	{
		errs	+= aio168_fsamp_ao_report(fd, -1, 1, i, &sps);
		total	+= sps;
	}

	if (verbose)
	{
		gsc_label("Overall AO Rate");
		gsc_label_long_comma(total);
		printf(" S/S\n");
		gsc_label_level_dec();
	}

	if (fsamp)
		fsamp[0]	= total;

	return(errs);
}


