// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO168/16aio168_Linux_2.x.x.x_DN/utils/util_irq0_sel.c $
// $Rev: 53661 $
// $Date: 2023-08-31 15:59:26 -0500 (Thu, 31 Aug 2023) $

// 16AIO168: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aio168_irq0_sel
*
*	Purpose:
*
*		Provide a visual wrapper for the AIO168_IOCTL_IRQ0_SEL service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int aio168_irq0_sel(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("IRQ 0 Select", index);

	ret		= aio168_ioctl(fd, AIO168_IOCTL_IRQ0_SEL, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AIO168_IRQ0_IDLE_INIT:

			ptr	= "Idle/Initialize Done";
			break;

		case AIO168_IRQ0_AUTOCAL_DONE:

			ptr	= "Autocalibrate Done";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


