// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO168/16aio168_Linux_2.x.x.x_DN/utils/util_query.c $
// $Rev: 53661 $
// $Date: 2023-08-31 15:59:26 -0500 (Thu, 31 Aug 2023) $

// 16AIO168: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aio168_query
*
*	Purpose:
*
*		Provide a visual wrapper for the AIO168_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int aio168_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;
	s32			tmp;

	switch (query)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Query Error (#%ld)", (long) query);
			break;

		case AIO168_QUERY_AUTOCAL_MS:	ptr	= "Autocal Duration";	break;
		case AIO168_QUERY_CHAN_AI_MAX:	ptr	= "AI Max Channels";	break;
		case AIO168_QUERY_CHAN_AO_MAX:	ptr	= "AO Max Channels";	break;
		case AIO168_QUERY_COUNT:		ptr	= "Query Options";		break;
		case AIO168_QUERY_DEVICE_TYPE:	ptr	= "Device Type";		break;
		case AIO168_QUERY_FIFO_SIZE_RX:	ptr	= "Rx FIFO Size";		break;
		case AIO168_QUERY_FIFO_SIZE_TX:	ptr	= "Tx FIFO Size";		break;
		case AIO168_QUERY_FREF_DEFAULT:	ptr	= "Fref Default";		break;
		case AIO168_QUERY_FSAMP_MAX:	ptr	= "Fsamp Max";			break;
		case AIO168_QUERY_FSAMP_MIN:	ptr	= "Fsamp Min";			break;
		case AIO168_QUERY_FGEN_MAX:		ptr	= "Fgen Max";			break;
		case AIO168_QUERY_FGEN_MIN:		ptr	= "Fgen Min";			break;
		case AIO168_QUERY_INIT_MS:		ptr	= "Init. Duration";		break;
		case AIO168_QUERY_NRATE_MAX:	ptr	= "Nrate Max";			break;
		case AIO168_QUERY_NRATE_MIN:	ptr	= "Nrate Min";			break;
	}

	if (verbose)
		gsc_label_index(ptr, index);

	ret		= aio168_ioctl(fd, AIO168_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	switch (query)
	{
		default:

			errs++;
			sprintf(buf, "%ld", (long) set);
			break;

		case AIO168_QUERY_CHAN_AI_MAX:
		case AIO168_QUERY_CHAN_AO_MAX:
		case AIO168_QUERY_COUNT:
		case AIO168_QUERY_NRATE_MAX:
		case AIO168_QUERY_NRATE_MIN:

			gsc_label_long_comma_buf(set, buf);
			break;

		case AIO168_QUERY_FGEN_MAX:
		case AIO168_QUERY_FGEN_MIN:
		case AIO168_QUERY_FREF_DEFAULT:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " Hz");
			break;

		case AIO168_QUERY_AUTOCAL_MS:
		case AIO168_QUERY_INIT_MS:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " ms");
			break;

		case AIO168_QUERY_FSAMP_MAX:
		case AIO168_QUERY_FSAMP_MIN:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " S/S");
			break;

		case AIO168_QUERY_FIFO_SIZE_RX:
		case AIO168_QUERY_FIFO_SIZE_TX:

			ptr	= "";
			tmp	= set;

			if ((tmp) && ((tmp % 1024) == 0))
			{
				ptr	= "K";
				tmp	/= 1024;
			}

			gsc_label_long_comma_buf(tmp, buf);
			strcat(buf, ptr);
			strcat(buf, " Words");
			break;

		case AIO168_QUERY_DEVICE_TYPE:

			if (set == GSC_DEV_TYPE_16AIO168)
			{
				strcpy(buf, "16AIO168");
			}
			else
			{
				errs++;
				sprintf(buf, "INVALID: %ld", (long) set);
			}

			break;
	}

	if (verbose)
	{
		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


