// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO168/16aio168_Linux_2.x.x.x_DN/utils/util_reg.c $
// $Rev: 53661 $
// $Date: 2023-08-31 15:59:26 -0500 (Thu, 31 Aug 2023) $

// 16AIO168: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	_GSC_REG(a)			"GSC " #a, AIO168_GSC_##a, 0, 0
#define	NAME_WIDTH			25

#define	FIELD_SHOW(beg,end,eol,lst,dsc)		\
			gsc_reg_field_show(width+10,NAME_WIDTH,value,(beg),(end),(eol),(lst),(dsc))



//*****************************************************************************
static int _bcr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	_aim[]	=
	{
		"Differential",
		"Single-Ended",
		"Zero Test",
		"+Vref Test",
		"Output Channel 0 Loopback",
		"Output Channel 1 Loopback",
		"Output Channel 2 Loopback",
		"Output Channel 3 Loopback",
		"Output Channel 4 Loopback",
		"Output Channel 5 Loopback",
		"Output Channel 6 Loopback",
		"Output Channel 7 Loopback",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved"
	};

	static const char*	_range[]	=
	{
		"+-2.5 Volts",
		"+-5 Volts",
		"+-10 Volts",
		"+-10 Volts"
	};

	static const char*	_disable[]	= { "Disabled",			"Enabled"			};
	static const char*	_fail[]		= { "Fail",		"Pass"		};
	static const char*	_format[]	= { "Twos Compliment",	"Offset Binary"		};
	static const char*	_idle[]		= { "Idle",				"Active"			};
	static const char*	_seq[]		= { "No (Sequential)",	"Yes (Synchronous)"	};

	FIELD_SHOW( 3,  0, 1, _aim,		"Analog Input Mode"		);
	FIELD_SHOW( 5,  4, 1, _range,	"Voltage Range"			);
	FIELD_SHOW( 6,  6, 1, _format,	"Data Format"			);
	FIELD_SHOW( 7,  7, 1, NULL,		"Reserved"				);
	FIELD_SHOW( 8,  8, 1, _seq,		"Simultaneous Outputs"	);
	FIELD_SHOW( 9,  9, 1, _disable,	"Output Bursting"		);
	FIELD_SHOW(10, 10, 1, _disable,	"Output Looping"		);
	FIELD_SHOW(11, 11, 1, _idle,	"Output Sync"			);
	FIELD_SHOW(12, 12, 1, _idle,	"Input Sync"			);
	FIELD_SHOW(13, 13, 1, _idle,	"Autocalibrate"			);
	FIELD_SHOW(14, 14, 1, _fail,	"Autocal Status"		);
	FIELD_SHOW(15, 15, 1, _idle,	"Initialize"			);
	FIELD_SHOW(31, 16, 1, NULL,		"Reserved"				);
	return(0);
}



//*****************************************************************************
static int _icr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	_irq0[]	=
	{
		"Idle/Initialize Done",
		"Autocalibration Done",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved"
	};

	static const char*	_irq1[]	=
	{
		"Idle/Disable",
		"Input Buffer Threshold Low-to-High",
		"Input Buffer Threshold High-to-Low",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved"
	};

	static const char*	_irq2[]	=
	{
		"Idle/Disable",
		"Output Buffer Threshold Low-to-High",
		"Output Buffer Threshold High-to-Low",
		"Output Burst Completed",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved"
	};

	static const char*	_idle[]	= { "Idle",	"Active"	};

	FIELD_SHOW( 2,  0, 1, _irq0,	"IRQ 0 Select"	);
	FIELD_SHOW( 3,  3, 1, _idle,	"IRQ 0 Status"	);
	FIELD_SHOW( 6,  4, 1, _irq1,	"IRQ 1 Select"	);
	FIELD_SHOW( 7,  7, 1, _idle,	"IRQ 1 Status"	);
	FIELD_SHOW(10,  8, 1, _irq2,	"IRQ 2 Select"	);
	FIELD_SHOW(11, 11, 1, _idle,	"IRQ 2 Status"	);
	FIELD_SHOW(31, 12, 1, NULL,		"Reserved"		);
	return(0);
}



//*****************************************************************************
static int _idbr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	_idle[]	= { "Idle",	"Active"	};

	FIELD_SHOW(15,  0, 1, NULL,		"Data"			);
	FIELD_SHOW(16, 16, 1, _idle,	"Channel Tag"	);
	FIELD_SHOW(31, 17, 1, NULL,		"Reserved"		);
	return(0);
}



//*****************************************************************************
static int _ibcr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	_idle[]	= { "Idle",	"Active"	};

	FIELD_SHOW(14,  0, 1, NULL,		"Threshold Value"	);
	FIELD_SHOW(15, 15, 1, _idle,	"Clear Buffer"		);
	FIELD_SHOW(16, 16, 1, _idle,	"Threshold Flag"	);
	FIELD_SHOW(31, 17, 1, NULL,		"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _rxgr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	_enable[]	= { "Enabled",	"Disabled"	};

	long	v;

	FIELD_SHOW(15,  0, 0, NULL,		"Nrate"				);
	v	= value & 0xFFFF;
	printf("0x%lX  (Fref / %ld)\n", v, v);

	FIELD_SHOW(16, 16, 1, _enable,	"Generator Disable"	);
	FIELD_SHOW(31, 17, 1, NULL,		"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _odbr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	_idle[]	= { "Idle",	"Active"	};

	FIELD_SHOW(15,  0, 1, NULL,		"Data"					);
	FIELD_SHOW(18, 16, 1, NULL,		"Output Channel Tag"	);
	FIELD_SHOW(19, 19, 1, _idle,	"Group End Flag"		);
	FIELD_SHOW(20, 20, 1, _idle,	"Burst End Flag"		);
	FIELD_SHOW(31, 21, 1, NULL,		"Reserved"				);
	return(0);
}



//*****************************************************************************
static int _obcr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	_idle[]	= { "Idle",	"Active"	};

	FIELD_SHOW(14,  0, 1, NULL,		"Threshold Level"	);
	FIELD_SHOW(15, 15, 1, _idle,	"Clear Buffer"		);
	FIELD_SHOW(16, 16, 1, _idle,	"Threshold Flag"	);
	FIELD_SHOW(31, 17, 1, NULL,		"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _sscr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	_size[]	=
	{
		"4 Channels (0-3)",
		"8 Channels (0-7)",
		"16 Channels (0-15)",
		"32 Channels (0-31)"
	};

	static const char*	_in_clk[]	=
	{
		"Rate-A Generator",
		"Rate-B Generator",
		"Ext Sync Input",
		"BCR Input Sync"
	};

	static const char*	_out_clk[]	=
	{
		"Rate-A Generator",
		"Rate-B Generator",
		"Ext Sync Input",
		"Disable"
	};

	static const char*	_out_sync[]	=
	{
		"Rate-A Generator",
		"Rate-B Generator",
		"Ext Sync Input",
		"BCR Output Sync"
	};

	static const char*	_ext_sync[]	=
	{
		"Input Scan Clock",
		"Output Sync",
		"Ext Sync Input",
		"Disabled"
	};

	static const char*	_master[]	= { "Master",	"Rate-A Generator"	};
	static const char*	_multi[]	= { "Multiple",	"Single"			};
	static const char*	_no[]		= { "No",		"Yes"				};

	long	field;

	FIELD_SHOW( 1,  0, 1, _size,		"Scan Size"					);
	FIELD_SHOW( 3,  2, 1, _in_clk,		"Input Clock Source"		);
	FIELD_SHOW( 5,  4, 1, _out_clk,		"Output Clock Source"		);
	FIELD_SHOW( 7,  6, 1, _out_sync,	"Output Sync Source"		);
	FIELD_SHOW( 9,  8, 1, _ext_sync,	"Ext Sync Output Source"	);
	FIELD_SHOW(10, 10, 1, _master,		"Rate-B Generator Source"	);
	FIELD_SHOW(11, 11, 1, _multi,		"Input Scanning Mode"		);

	FIELD_SHOW(16, 12, 0, NULL,			"Single-Channel Select"		);
	field	= (value >> 12) & 0x1F;
	printf("Channel # %ld\n", (long) field);

	FIELD_SHOW(17, 17, 1, _no,			"Two Channel Scan"			);
	FIELD_SHOW(31, 18, 1, NULL,			"Reserved"					);
	return(0);
}



//*****************************************************************************
static int _docr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	_low[]	= { "0, Low",	"1, High"	};

	FIELD_SHOW( 0,  0, 1, _low,	"Digital Output 0"	);
	FIELD_SHOW( 1,  1, 1, _low,	"Digital Output 1"	);
	FIELD_SHOW( 2,  2, 1, _low,	"Digital Output 2"	);
	FIELD_SHOW( 3,  3, 1, _low,	"Digital Output 3"	);
	FIELD_SHOW(31,  4, 1, NULL,	"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _frr_decode(int fd, int supported, u32 value, int width)
{
	long	field;

	FIELD_SHOW(15,  0, 0, NULL,	"Firmware Revision"	);
	field	= value & 0xFFFF;
	printf("%04lX\n", field);

	FIELD_SHOW(31, 16, 1, NULL,	"Reserved"			);
	return(0);
}



// variables ******************************************************************

static gsc_reg_def_t	_gsc[]	=
{

	{ _GSC_REG(BCR),	0,	_bcr_decode,	"Board Control Register"			},
	{ _GSC_REG(ICR),	0,	_icr_decode,	"Interrupt Control Register"		},
	{ _GSC_REG(IDBR),	0,	_idbr_decode,	"Input Data Buffer Register"		},
	{ _GSC_REG(IBCR),	0,	_ibcr_decode,	"Input Buffer Control Register"		},
	{ _GSC_REG(RAGR),	0,	_rxgr_decode,	"Rate Generator A Register"			},
	{ _GSC_REG(RBGR),	0,	_rxgr_decode,	"Rate Generator B Register"			},
	{ _GSC_REG(ODBR),	0,	_odbr_decode,	"Output Data Buffer Register"		},
	{ _GSC_REG(OBCR),	0,	_obcr_decode,	"Output Buffer Control Register"	},
	{ _GSC_REG(SSCR),	0,	_sscr_decode,	"Scan and Sync Control Register"	},
	{ _GSC_REG(DOCR),	0,	_docr_decode,	"Digital Output Control Register"	},
	{ _GSC_REG(FRR),	0,	_frr_decode,	"Firmware Revision Register"		},
	{ _GSC_REG(AVR),	0,	NULL,			"Autocal Values Register"			},
	{ NULL	}
};



//*****************************************************************************
static const gsc_reg_def_t* _find_reg(u32 reg, const gsc_reg_def_t* list)
{
	const gsc_reg_def_t*	def	= NULL;
	int						i;

	for (i = 0; list[i].name; i++)
	{
		if (reg == list[i].reg)
		{
			def	= &list[i];
			break;
		}
	}

	return(def);
}



/******************************************************************************
*
*	Function:	aio168_reg_get_def_id
*
*	Purpose:
*
*		Retrieve the register definition structure given the register id.
*
*	Arguments:
*
*		reg		The id of the register to access.
*
*	Returned:
*
*		NULL	The register id wasn't found.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* aio168_reg_get_def_id(u32 reg)
{
	const gsc_reg_def_t*	def;

	def	= _find_reg(reg, _gsc);
	return(def);
}



/******************************************************************************
*
*	Function:	aio168_reg_get_def_index
*
*	Purpose:
*
*		Retrieve the register definition structure based on an index.
*
*	Arguments:
*
*		index	The index of the register to access.
*
*	Returned:
*
*		NULL	The index doesn't correspond to a known register.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* aio168_reg_get_def_index(int index)
{
	const gsc_reg_def_t*	def;

	if (index < 0)
		def	= NULL;
	else if (index >= (SIZEOF_ARRAY(_gsc) - 1))
		def	= NULL;
	else
		def	= &_gsc[index];

	return(def);
}



/******************************************************************************
*
*	Function:	aio168_reg_get_desc
*
*	Purpose:
*
*		Retrieve the description of the specified register.
*
*	Arguments:
*
*		reg		The register whose description is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* aio168_reg_get_desc(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				desc;

	def	= _find_reg(reg, _gsc);

	if (def)
		desc	= def->desc;
	else
		desc	= "UNKNOWN";

	return(desc);
}



/******************************************************************************
*
*	Function:	aio168_reg_get_name
*
*	Purpose:
*
*		Retrieve the name of the specified register.
*
*	Arguments:
*
*		reg		The register whose name is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* aio168_reg_get_name(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				name;

	def	= _find_reg(reg, _gsc);

	if (def)
		name	= def->name;
	else
		name	= "UNKNOWN";

	return(name);
}



/******************************************************************************
*
*	Function:	aio168_reg_list
*
*	Purpose:
*
*		List the GSC registers and their values.
*
*	Arguments:
*
*		fd		The handle to access the device.
*
*		detail	List the register details?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aio168_reg_list(int fd, int detail)
{
	int	errs;

	errs	= gsc_reg_list(fd, _gsc, detail, aio168_reg_read);
	return(errs);
}



